// Vikunja is a to-do list application to facilitate your life.
// Copyright 2018-2020 Vikunja and contributors. All rights reserved.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

package models

import (
	"code.vikunja.io/api/pkg/user"
	"code.vikunja.io/web"
	"time"
)

// SavedFilter represents a saved bunch of filters
type SavedFilter struct {
	// The unique numeric id of this saved filter
	ID int64 `xorm:"autoincr not null unique pk" json:"id"`
	// The actual filters this filter contains
	Filters *TaskCollection `xorm:"JSON not null" json:"filters"`
	// The title of the filter.
	Title string `xorm:"varchar(250) not null" json:"title" valid:"required,runelength(1|250)" minLength:"1" maxLength:"250"`
	// The description of the filter
	Description string `xorm:"longtext null" json:"description"`
	OwnerID     int64  `xorm:"int(11) not null INDEX" json:"-"`

	// The user who owns this filter
	Owner *user.User `xorm:"-" json:"owner" valid:"-"`

	// A timestamp when this filter was created. You cannot change this value.
	Created time.Time `xorm:"created not null" json:"created"`
	// A timestamp when this filter was last updated. You cannot change this value.
	Updated time.Time `xorm:"updated not null" json:"updated"`

	web.CRUDable `xorm:"-" json:"-"`
	web.Rights   `xorm:"-" json:"-"`
}

// TableName returns a better table name for saved filters
func (s *SavedFilter) TableName() string {
	return "saved_filters"
}

func (s *SavedFilter) getTaskCollection() *TaskCollection {
	// We're resetting the listID to return tasks from all lists
	s.Filters.ListID = 0
	return s.Filters
}

func getSavedFilterIDFromListID(listID int64) int64 {
	// We get the id of the saved filter by multiplying the ListID with -1 and subtracting one
	return listID*-1 - 1
}

func (s *SavedFilter) Create(auth web.Auth) error {
	s.OwnerID = auth.GetID()
	_, err := x.Insert(s)
	return err
}

func getSavedFilterSimpleByID(id int64) (s *SavedFilter, err error) {
	_, err = x.Where("id = ?", id).Get(s)
	return
}

func (s *SavedFilter) ReadOne() error {
	// s already contains almost the full saved filter from the rights check, we only need to add the user
	u, err := user.GetUserByID(s.OwnerID)
	s.Owner = u
	return err
}

// ReadAll returns all tasks of a saved filter.
// Since saved filters are shown in a pseudo-namespace, we can use this function to get all tasks
func (s *SavedFilter) ReadAll(auth web.Auth, search string, page int, perPage int) (result interface{}, resultCount int, numberOfTotalItems int64, err error) {
	return
}

func (s *SavedFilter) Update() error {
	_, err := x.Where("id = ?", s.ID).Update(s)
	return err
}

func (s *SavedFilter) Delete() error {
	_, err := x.Where("id = ?", s.ID).Delete(s)
	return err
}
